//-----------------------------------------------------------------------------
// Component Name: WndActImg
// Author:  Alex G.
// Date:    2002-03-02
// Purpose: Extension of standard TImage. It can react on clicks and drag
//          action the same way as form border or form caption does.
//          It gives possibility to resize/move Form by clicking and dragging
//          TWinActImg object.
//-----------------------------------------------------------------------------
unit WndActImg;

interface

uses
  Classes, Controls, Windows, Messages, ShellAPI, ExtCtrls;

type
  TWndAction   = procedure (Sender: TObject; Button: TMouseButton;
                            Shift: TShiftState; X, Y: Integer) of object;
  TClickTargets =  (atCaption,            //Window caption (move window)
                    atBorderTop,          //Top border (resize up/down)
                    atBorderRight,        //Right border (resize left/right)
                    atBorderBottom,       //Bottom border (resize up/down)
                    atBorderLeft,         //Left border (resize left/right)
                    atCornerTopRight,
                    atCornerTopLeft,
                    atCornerBottonRight,
                    atCornerBottomLeft,
                    atNone);

  TWndActImage = class(TImage)
    private
      FClickTarget    : TClickTargets;
      FShowWndContent : boolean;
      FWindowAction   : TWndAction;
      //--- Action
      procedure DoClickAction (Sender: TObject; Button: TMouseButton;
                               Shift: TShiftState; X, Y: Integer);

    public
      constructor Create(AOwner : TComponent); override;
      destructor  Destroy(); override;

    published
      //--- Proprties
      property ClickTarget: TClickTargets
               read FClickTarget write FClickTarget default atCaption;
      property ShowWndContent: boolean
               read FShowWndContent write FShowWndContent default false;
  end;

procedure Register;

implementation


//-----------------------------------------------------------------------------
// Procedure: Register
// Date:      2002-03-02
// Purpose:   Register visual component in "Additional" tab
//-----------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('Additional', [TWndActImage]);
end;

//-----------------------------------------------------------------------------
// Procedure: Create
// Date:      2002-03-02
// Purpose:   Init wariables on create
//-----------------------------------------------------------------------------
constructor TWndActImage.Create;
begin
  inherited Create(AOwner);
  FWindowAction := DoClickAction;
  OnMouseDown   := FWindowAction
end;

//-----------------------------------------------------------------------------
// Procedure: Destroy
// Date:      2002-03-02
// Purpose:   Call TImage.Destroy
//-----------------------------------------------------------------------------
destructor TWndActImage.Destroy;
begin
  inherited Destroy();
end;

//-----------------------------------------------------------------------------
// Procedure: DoClickAction
// Date:      2002-03-02
// Purpose:   Forward "click" message to apropriate part of the form border
//-----------------------------------------------------------------------------
procedure TWndActImage.DoClickAction;
var
  b : boolean;
  ht: integer;
begin
  SystemParametersInfo(SPI_GETDRAGFULLWINDOWS, 0, @b, 0);
  SystemParametersInfo(SPI_SETDRAGFULLWINDOWS,Integer(FShowWndContent),nil,0);
  ReleaseCapture;
  ht := HTNOWHERE;
  case FClickTarget of
    atCaption          : ht := HTCAPTION;
    atBorderTop        : ht := HTTOP;
    atBorderRight      : ht := HTRIGHT;
    atBorderBottom     : ht := HTBOTTOM;
    atBorderLeft       : ht := HTLEFT;
    atCornerTopRight   : ht := HTTOPRIGHT;
    atCornerTopLeft    : ht := HTTOPLEFT;
    atCornerBottonRight: ht := HTBOTTOMRIGHT;
    atCornerBottomLeft : ht := HTBOTTOMLEFT;
    atNone             : ht := HTNOWHERE;
  end;
  SendMessage(TWinControl(Owner).Handle, WM_NCLBUTTONDOWN, ht, 0);
  SystemParametersInfo(SPI_SETDRAGFULLWINDOWS,Integer(FShowWndContent),nil,0);
  inherited;
end;

end.
